package ASCOPE::Email::Graph;
use strict;

# $Id: Graph.pm,v 1.5 2007/12/26 18:51:14 asc Exp $

$ASCOPE::Email::Graph::VERSION = '0.2';

=head1 NAME

ASCOPE::Email::Graph - Aaron's widget for graphing email correspondence.

=head1 SYNOPSIS

  my $graph = ASCOPE::Email::Graph->new();

  $graph->add_mbox("/path/to/mbox");

  $graph->add_equivalencies(["me\@work.net", "me\@home.net", "me\@gmail.com"], "icanhas\@email.us");
  $graph->add_equivalencies(["buddy\@cheezburger.com", "bud.dy\@ohhai.com"]);
  $graph->add_equivalencies(["girly\@cheezburger.com", "grrrr.ly\@ohrly.com"]);

  $graph->add_cluster(["buddy\@cheezburger.com", "bud.dy\@ohhai.com",
                       "girly\@cheezburger.com", "grrrr.ly\@ohrly.com"],
                       "cheezburger");

  $graph->as_png("/path/to/graph.png");

=head1 DESCRIPTION

Aaron's widget for graphing email correspondence.

=cut

use Email::Folder;
use Email::Find;

use Color::Calc;

use GraphViz;
use Digest::SHA1 qw(sha1_hex);

sub import {
    my $self = shift;

    my @valid = qw(as_canon
		   as_text
		   as_ps
		   as_hpgl
		   as_pcl
		   as_mif
		   as_pic
		   as_gd
		   as_gd2
		   as_jpeg
		   as_png
		   as_wbmp
		   as_cmap
		   as_ismap
		   as_imap
		   as_vrml
		   as_vtx
		   as_mp
		   as_fig
		   as_svg
		   as_plain);

    foreach my $meth (@valid) {
	no strict "refs";

	*$meth = sub {
	    my $self    = shift;
	    my $outfile = shift;

	    my $gv = $self->_graph();
	    $gv->$meth($outfile);
	};
    }

    return 1;
}

=head1 PACKAGE METHODS

=cut

=head2 __PACKAGE__->new(\%args)

=over 4

=item * B<colour>

String; a valid X11 color name or a RRGGBB hex string.

Base colour used to fill individual nodes; each node will be filled
using a percentage of this colour based on its weight.

Default is I<grey>

=item * B<color>

No comment.

=item * B<shape>

String; The shape to use for each each. From the GraphViz docs :

 This can be one of: 'record', 'plaintext', 'ellipse', 'circle',
 'egg', 'triangle', 'box', 'diamond', 'trapez-ium', 'parallelogram',
 'house', 'hexagon', 'octagon' 

Defaut is B<box>

=item * B<fontsize>

Int.

Font size used to write email addresses.

Default is I<6>

=item * B<hash>

Boolean.

Hash email addresses using I<Digest::SHA1>. Addresses will be formatted as
FOAF mmbox_sha1 strings, as in : 

 sha1_hex("mailto:" . $address)

Default is I<false>.

=item * B<obscure>

Do not print any labels for nodes. 

Default is I<false>.

=back

Returns a I<ASCOPE::Email::Graph> object. Woot!

=cut

sub new {
    my $pkg  = shift;
    my $args = shift;

    my $self = {"_colour"   => ($args->{colour}   || $args->{color} || "grey"),
                "_shape"    => ($args->{shape}    || 'box'),
		"_fontsize" => ($args->{fontsize} || '6'),
		"_hash"     => ($args->{hash}  || 0),
                "_layout" => ($args->{'layout'} || 'neato'),
                "_obscure" => ($args->{'obscure'} || 0),
		"_nodes" => {},
		"_rels" => {},
                "_hashes" => {},
                "_aliases" => {},
                "_clusters" => {},
               };

    return bless $self,$pkg;
}

=head1 OBJECT METHODS

=cut

=head2 $obj->add_mbox($path)

=cut

sub add_mbox {
    my $self = shift;
    my $path = shift;

    my $mbox = Email::Folder->new($path);

    if (! $mbox) {
	warn "Failed to create mbox for $path : $!\n";
	return 0;
    }
    
    while (my $msg = $mbox->next_message()) {
	$self->_address($msg);
    }
    
    return 1;
}

=head2 $obj->add_equivalencies(\@list, $key)

Treat all the email addresses in I<@list> as the same, assuming their
value is I<$key>.

If I<$key> is not defined then use the first address defined in I<@list>.

=cut

sub add_equivalencies {
        my $self = shift;
        my $list = shift;
        my $key = shift;

        if (! $key){
                $key = $list->[0];
        }

        foreach my $addr (@$list){
                $self->{'_aliases'}->{$addr} = $key;
        }
}

=head2 $obj->add_cluster(\@list, $key)

Collect a set of email addresses in I<@list> as the cluster, assuming the
label is I<$key>.

If I<$key> is not defined then use the first address defined in I<@list>.

=cut

sub add_cluster {
        my $self = shift;
        my $list = shift;
        my $key = shift;

        if (! $key){
                $key = $list->[0];
        }

        foreach my $addr (@$list){
                $self->{'_clusters'}->{$addr} = $key;
        }
}

=head2 $obj->as_canon($path);

=head2 $obj->as_text($path);

=head2 $obj->as_ps($path);

=head2 $obj->as_hpgl($path);

=head2 $obj->as_pcl($path);

=head2 $obj->as_mif($path);

=head2 $obj->as_pic($path);

=head2 $obj->as_gd($path);

=head2 $obj->as_gd2($path);

=head2 $obj->as_jpeg($path);

=head2 $obj->as_png($path);

=head2 $obj->as_wbmp($path);

=head2 $obj->as_cmap($path);

=head2 $obj->as_ismap($path);

=head2 $obj->as_imap($path);

=head2 $obj->as_vrml($path);

=head2 $obj->as_vtx($path);

=head2 $obj->as_mp($path);

=head2 $obj->as_fig($path);

=head2 $obj->as_svg($path);

=head2 $obj->as_plain($path);

=cut

sub _hash {
        my $self = shift;
        my $address = shift;

        if (! exists($self->{'_hashes'}->{$address})){
                $self->{'_hashes'}->{$address} = sha1_hex("mailto:$address");
        }

        return $self->{'_hashes'}->{$address};
}

sub _address {
    my $self = shift;
    my $msg  = shift;

    my %addr = (from => [],
		to   => [],
		cc   => []);

    #

    foreach my $type ("from","to","cc") {
	
	foreach my $string ($msg->header($type)) {
	    
	    &find_emails(\$string, sub {
		my ($email, $orig_email) = @_;
		my $address = $email->format();

                if (my $alias = $self->_alias($address)){
                        $address = $alias;
                }

		if ($self->{'_hash'}) {
                        $address = $self->_hash($address);
		}

		push @{$addr{$type}}, $address;
		$self->{'_nodes'}->{ $address } ++;
	    });
	}
    }
    
    #

    foreach my $from (@{$addr{"from"}}) {
	
	foreach my $to (@{$addr{"to"}}) {
	    $self->{'_rels'}->{ $from }->{ $to } ++;
	}
	
	foreach my $cc (@{$addr{"cc"}}) {
	    $self->{'_rels'}->{ $from }->{ $cc } ++;
	}
    }
    
    #
    
    return 1;
}

sub _alias {
        my $self = shift;
        my $address = shift;

        if (exists($self->{'_aliases'}->{$address})){
                return $self->{'_aliases'}->{$address};
        }

        return undef;
}

sub _cluster {
        my $self = shift;
        my $address = shift;

        if (exists($self->{'_clusters'}->{$address})){
                return $self->{'_clusters'}->{$address};
        }

        return undef;
}

sub _graph {
    my $self = shift;
  
    my $g = GraphViz->new('directed' => 1,
                          'overlap' => 'scale',
                          'layout' => $self->{'_layout'},
			  'concentrate' => 1);

    my $first = (sort {$b <=> $a } values %{$self->{'_nodes'}})[0];

    #

    foreach my $who (keys %{$self->{'_nodes'}}) {
	
       	my $colour = $self->{'_colour'};
	my $alpha  = undef;

	my $weight = $self->{'_nodes'}->{$who};

	if ($weight != $first) {
	    $alpha  = 1 - ($weight / $first);
	    $colour = sprintf("#%s",color_light_hex($colour,
						    $alpha));
	}

        my $scale = ($self->{'_hash'}) ? 80 : 50;
        my $width = int(($self->{'_fontsize'} * length($who)) / 80);
        
        # print STDERR "[$who] $width\n";
	# print "[$who] $colour ($weight) ($alpha)\n";

        my %node_args = (
                         'fontsize' => $self->{'_fontsize'},
                         'width' => $width,
                         'style' => "filled",
                         'shape' => $self->{'_shape'},
                         'fillcolor' => $colour
                        );

        if ($self->{'_obscure'}){
                $node_args{'label'} = ' ';
        }

        if (my $cluster = $self->_cluster($who)){

                my %cluster_args = (
                                    name => $cluster,
                                    style     => 'bold',
                                    color => 'red',
                                    fontsize  => $self->{'_fontsize'},
                                   );

                $node_args{'cluster'} = \%cluster_args;
        }
        
	$g->add_node($who, %node_args);
    }

    #

    foreach my $from (keys %{$self->{'_rels'}}) {
	
	foreach my $to (keys %{$self->{'_rels'}->{$from}}) {
	    
	    $g->add_edge($from    => $to,
			 minlen   => 2,
			 weight   => $self->{'_nodes'}->{$from});
	    
	}
    }
 
    #

    return $g;
}

sub DESTROY {}

=head1 VERSION

0.2

=head1 DATE

$Date: 2007/12/26 18:51:14 $

=head1 AUTHOR

Aaron Straup Cope E<lt>ascope@cpan.orgE<gt>

=head1 SEE ALSO 

L<http://www.aaronland.info/weblog/2004/06/07/5527/>

L<http://www.aaronland.info/weblog/2007/12/20/castles/#rocks>

L<http://www.tom-carden.co.uk/2007/11/08/sticks-and-rocks-illustrating-the-problem/>

L<GraphViz>

L<Email::Folder>

=head1 LICENSE

Copyright (c) 2004-2007 Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under
the same terms as Perl itself.

=cut

return 1;

__END__

